{==============================================================================}
{                                                                              }
{  ProjectName  :                                                              }
{  Unity        : INCLUDE                                                      }
{  Name and date: ErrorTextDeclarations.INC 06-05-99                           }
{  Writer       : PROCES-DATA A/S                                              }
{  Modification :                                                              }
{                                                                              }
{------------------------------------------------------------------------------}
{ Error text declarations for the standard error handling procedures.          }
{ The error texts are defined as constants in two languages, English and       }
{ Danish. A variable is declared as the same type as the error texts. When a   }
{ language is selected, the language constant is copied to the variable and    }
{ this variable is used in the error handling procedures.                      }
{------------------------------------------------------------------------------}

TYPE
  TErrorString = String[40];

  TErrorTypes = ( NoResponse,
                  TimeOut,
                  TooBusy,
                  WaitTooLong,
                  BufferFullOrEmpty,
                  DataFormatError,
                  SWNOError,
                  NodeAddressError,
                  WriteProtection,
                  InfoLengthError,
                  InstructionError,
                  ErrorDetectFailure,
                  OverrunFramingError,
                  NetShortcircuit,
                  PortNotMaster,
                  OutOfSync,
                  RS232HandshakeError,
                  TransError,
                  Err,
                  servicech0,
                  internal_reset,
                  external_reset,
                  ROM_error,
                  RAM_error,
                  watchdog_error,
                  EEPROM_error,
                  Battery_low,
                  channelerror,
                  digital_IO,
                  module_error,
                  overload,
                  underload,
                  external_shortcircuit,
                  internal_shortcircuit,
                  internal_disconnection,
                  feedback_error,
                  programming_error,
                  analog_input,
                  signal_high,
                  signal_low,
                  high_alarm,
                  low_alarm,
                  internal_error,
                  analog_output,
                  no_load,
                  dataerror_on_setpoint,
                  output_high,
                  output_low,
                  dataerror_on_signal,
                  calculatorModule,
                  weightModule,
                  external_error,
                  prgram,
                  no_prog_after_reset,
                  no_program_code,
                  wrong_interpr_vers,
                  error_in_program,
                  Power,
                  current_high,
                  voltage_high,
                  symmetry_error,
                  Communication,
                  inputbuffer_overrun,
                  parity_or_framing_error,
                  reservation_timeout,
                  config_err_datamode_param,
                  config_err_chconfig,
                  config_err_baudrate,
                  config_err_nodeparam,
                  reset_internal_error,
                  reset_via_PNET,
                  reset_powercut,
                  FlowTransmitter,
                  PROM_storage_error,
                  RAM_storage_error,
                  program_execution_error,
                  RAM_or_EEPROM_error,
                  temperature_connec_error,
                  magnetic_coil_disconnected,
                  magnetic_coil_shortcircuited,
                  shunt_defect,
                  temperature_Greater_than_max,
                  temperature_Greater_than_100C,
                  overrun_volume_counter_2,
                  overrun_volume_counter_1,
                  input_active,
                  flow_Greater_than_max_or_pipe_empty,
                  flow_Greater_than_max,
                  overflow_output_2,
                  overflow_output_3,
                  UnKnownError);


TErrorTextLanguage = (English, Danish); (* possible languages to select *)

TLanguageErrorText = ARRAY [TErrorTypes] OF TErrorString;

CONST
  EnglishTextsConstArr = TLanguageErrorText([NoResponse]: 'No Response',
                                            [TimeOut]: 'Time Out',
                                            [TooBusy]: 'Too Busy',
                                            [WaitTooLong]: 'Wait Too Long',
                                            [BufferFullOrEmpty]: 'Buffer full/empty',
                                            [DataFormatError]: 'Data Format Error',
                                            [SWNOError]: 'SWNO Error',
                                            [NodeAddressError]: 'Node Address Error',
                                            [WriteProtection]: 'Write Protection',
                                            [InfoLengthError]: 'Info Length Error',
                                            [InstructionError]: 'Instruction Error',
                                            [ErrorDetectFailure]: 'Error Detect Failure',
                                            [OverrunFramingError]: 'Overrun Framing Error',
                                            [NetShortcircuit]: 'Net Shortcurcuit',
                                            [PortNotMaster]: 'Port_Not_Master',
                                            [OutOfSync]: 'Other Error',
                                            [RS232HandshakeError]: 'General error',
                                            [TransError]: 'Trans. Error',
                                            [Err]: 'Error',
                                            [servicech0]: 'service',
                                            [internal_reset]: 'power failure',
                                            [external_reset]: 'reset via P-NET',
                                            [ROM_error]: 'ROM error',
                                            [RAM_error]: 'RAM error',
                                            [watchdog_error]: 'watchdog error',
                                            [EEPROM_error]: 'EEPROM error',
                                            [Battery_low]: 'Battery low',
                                            [channelerror]: 'channelerror',
                                            [digital_IO]: 'digital I/O',
                                            [module_error]: 'module error',
                                            [overload]: 'overload',
                                            [underload]: 'underload',
                                            [external_shortcircuit]: 'external shortcircuit',
                                            [internal_shortcircuit]: 'internal shortcircuit',
                                            [internal_disconnection]: 'internal disconnection',
                                            [feedback_error]: 'feed-back error',
                                            [programming_error]: 'programming error',
                                            [analog_input]: 'analog input',
                                            [signal_high]: 'signal high',
                                            [signal_low]: 'signal low',
                                            [high_alarm]: 'high alarm',
                                            [low_alarm]: 'low alarm',
                                            [internal_error]: 'internal error',
                                            [analog_output]: 'analog output',
                                            [no_load]: 'no load',
                                            [dataerror_on_setpoint]: 'dataerror on setpoint',
                                            [output_high]: 'output high',
                                            [output_low]: 'output low',
                                            [dataerror_on_signal]: 'dataerror on signal',
                                            [calculatorModule]: 'calculator',
                                            [weightModule]: 'weight',
                                            [external_error]: 'external error',
                                            [prgram]: 'program',
                                            [no_prog_after_reset]: 'no prog. after reset',
                                            [no_program_code]: 'no program code',
                                            [wrong_interpr_vers]: 'wrong interpr. vers.',
                                            [error_in_program]: 'error in program',
                                            [Power]: 'Power',
                                            [current_high]: 'current high',
                                            [voltage_high]: 'voltage high',
                                            [symmetry_error]: 'symmetry error',
                                            [Communication]: 'Communication',
                                            [inputbuffer_overrun]: 'inputbuffer overrun',
                                            [parity_or_framing_error]: 'parity/framing error',
                                            [reservation_timeout]: 'reservation timeout',
                                            [config_err_datamode_param]: 'config err. datamode param',
                                            [config_err_chconfig]: 'config err. chconfig',
                                            [config_err_baudrate]: 'config err. baudrate',
                                            [config_err_nodeparam]: 'config err. nodeparam',
                                            [reset_internal_error]: 'reset - internal error',
                                            [reset_via_PNET]: 'reset - via P-NET',
                                            [reset_powercut]: 'reset - power cut',
                                            [FlowTransmitter]: 'FlowTransmitter',
                                            [PROM_storage_error]: 'PROM storage error',
                                            [RAM_storage_error]: 'RAM storage error',
                                            [program_execution_error]: 'program execution error',
                                            [RAM_or_EEPROM_error]: 'RAM/EEPROM error',
                                            [temperature_connec_error]: 'temperature connec. error',
                                            [magnetic_coil_disconnected]: 'magnetic coil disconnected',
                                            [magnetic_coil_shortcircuited]: 'magnetic coil short-circuited',
                                            [shunt_defect]: 'shunt defect',
                                            [temperature_Greater_than_max]: 'temperature >> max',
                                            [temperature_Greater_than_100C]: 'temperature > 100C/266F',
                                            [overrun_volume_counter_2]: 'overrun volume counter 2',
                                            [overrun_volume_counter_1]: 'overrun volume counter 1',
                                            [input_active]: 'input active',
                                            [flow_greater_than_max_or_pipe_empty]: 'flow >> max / pipe empty',
                                            [flow_greater_than_max]: 'flow > max',
                                            [overflow_output_2]: 'overflow output 2',
                                            [overflow_output_3]: 'overflow output 3',
                                            [UnKnownError]: 'Unknown Error') [ MIBProperties = MIB_InVisible ];

  DanishTextsConstArr = TLanguageErrorText( [NoResponse]: 'Ingen svar',
                                            [TimeOut]: 'Time out fejl',
                                            [TooBusy]: 'Optaget',
                                            [WaitTooLong]: 'Ventet for lnge',
                                            [BufferFullOrEmpty]: 'Buffer fuld/tom',
                                            [DataFormatError]: 'Forkert data format',
                                            [SWNOError]: 'SWNO fejl',
                                            [NodeAddressError]: 'Node adresse fejl',
                                            [WriteProtection]: 'Skrivebeskyttet',
                                            [InfoLengthError]: 'Fejl i info lngde',
                                            [InstructionError]: 'Forkert instruction',
                                            [ErrorDetectFailure]: 'Fejl i fejlhndtering',
                                            [OverrunFramingError]: 'Overlb fejl',
                                            [NetShortCircuit]: 'NET kortsluttet',
                                            [PortNotMaster]: 'Port ikke master',
                                            [OutOfSync]: 'Ude af synk.',
                                            [RS232HandshakeError]: 'RS232 Handshake fejl',
                                            [TransError]: 'Trans. fejl',
                                            [Err]: 'Fejl',
                                            [servicech0]: 'service',
                                            [internal_reset]: 'strmsvigt',
                                            [external_reset]: 'reset via P-NET',
                                            [ROM_error]: 'ROM fejl',
                                            [RAM_error]: 'RAM fejl',
                                            [watchdog_error]: 'watchdog fejl',
                                            [EEPROM_error]: 'EEPROM fejl',
                                            [Battery_low]: 'Batteri lavt',
                                            [channelerror]: 'Kanal fejl',
                                            [digital_IO]: 'digital I/O',
                                            [module_error]: 'modul fejl',
                                            [overload]: 'overbelast.',
                                            [underload]: 'underbelast.',
                                            [external_shortcircuit]: 'ekstern kortslut.',
                                            [internal_shortcircuit]: 'intern kortslut.',
                                            [internal_disconnection]: 'intern frakobling',
                                            [feedback_error]: 'Tilbagemeldings fejl',
                                            [programming_error]: 'programmerings fejl',
                                            [analog_input]: 'analog input',
                                            [signal_high]: 'signal hjt',
                                            [signal_low]: 'signal lavt',
                                            [high_alarm]: 'hj alarm',
                                            [low_alarm]: 'lav alarm',
                                            [internal_error]: 'intern fejl',
                                            [analog_output]: 'analog output',
                                            [no_load]: 'Ingen belastning',
                                            [dataerror_on_setpoint]: 'datafejl p setpunkt',
                                            [output_high]: 'output hjt',
                                            [output_low]: 'output lavt',
                                            [dataerror_on_signal]: 'datafejl p signal',
                                            [calculatorModule]: 'calculator',
                                            [weightModule]: 'weight',
                                            [external_error]: 'ekstern fejl',
                                            [prgram]: 'program',
                                            [no_prog_after_reset]: 'Ingen prg. efter reset',
                                            [no_program_code]: 'Ingen program kode',
                                            [wrong_interpr_vers]: 'Forketr fortokler vers.',
                                            [error_in_program]: 'fejl i program',
                                            [Power]: 'Power',
                                            [current_high]: 'Strm hj',
                                            [voltage_high]: 'Spnding hj',
                                            [symmetry_error]: 'symmetri fejl',
                                            [Communication]: 'Communication',
                                            [inputbuffer_overrun]: 'inputbuffer overlb',
                                            [parity_or_framing_error]: 'paritets fejl',
                                            [reservation_timeout]: 'reservation timeout',
                                            [config_err_datamode_param]: 'config fejl datamode param',
                                            [config_err_chconfig]: 'config fejl chconfig',
                                            [config_err_baudrate]: 'config fejl baudrate',
                                            [config_err_nodeparam]: 'config fejl. nodeparam',
                                            [reset_internal_error]: 'reset pga. intern fejl',
                                            [reset_via_PNET]: 'reset pga. $FF i fejl via P-NET',
                                            [reset_powercut]: 'reset pga. strmsvigt',
                                            [FlowTransmitter]: 'FlowTransmitter',
                                            [PROM_storage_error]: 'Fejl i programlager (PROM)',
                                            [RAM_storage_error]: 'Fejl i datalager (RAM)',
                                            [program_execution_error]: 'Fejl i programafvikling',
                                            [RAM_or_EEPROM_error]: 'RAM/EEPROM fejl',
                                            [temperature_connec_error]: 'temperatur connec. fejl',
                                            [magnetic_coil_disconnected]: 'magnetspolen forkert forbundet',
                                            [magnetic_coil_shortcircuited]: 'magnetspolen kortsluttet',
                                            [shunt_defect]: 'shunt i mlehoved defekt',
                                            [temperature_Greater_than_max]: 'temperatur >> max',
                                            [temperature_Greater_than_100C]: 'temperatur > 100C/266F',
                                            [overrun_volume_counter_2]: 'overlb volume tller 2',
                                            [overrun_volume_counter_1]: 'overlb volume tller 1',
                                            [input_active]: 'indgang aktiveret',
                                            [flow_greater_than_max_or_pipe_empty]: 'flow >> max / mlerr tomt',
                                            [flow_greater_than_max]: 'flow > max',
                                            [overflow_output_2]: 'overflow output 2',
                                            [overflow_output_3]: 'overflow output 3',
                                            [UnKnownError]: 'Ukendt Fejl') [ MIBProperties = MIB_InVisible ];
  
VAR 
  ErrorTextLanguage: TErrorTextLanguage;
                          