{==============================================================================}
{                                                                              }
{  ProjectName  : PD4000                                                       }
{  Unity        : INCLUDE                                                      }
{  Name and date: When_Error.INC 06-05-99                                      }
{  Writer       : PROCES-DATA A/S                                              }
{  Modification :                                                              }
{                                                                              }
{------------------------------------------------------------------------------}
{ Standard error handling procedures for PD4000.                               }
{ The procedures can decode the errors that may occur on the P-NET and in      }
{ interface modules. The procedures return a string with the error information }
{ in clear text. The language of the error text depends on the included        }
{ language files.                                                              }
{ The format for the error string is as follows:                               }
{     <module-NAME> <channel-NAME> <channel-type> ch:<channel-no>: <error-type>}
{ where <module-NAME> is the NAME string defined for the module                }
{       <channel-NAME> is the NAME string defined for the channel              }
{       <channel-type> is a pre-defined text that specifies the channel type   }
{       <channel-no> is the channel number within the module                   }
{       <error-type> is a pre-defined text that specifies the type of error    }
{                                                                              }
{ e.g.  My UPI first channel digital I/O_01: programming error                 }
{ Handles conversion from New ErrorcodeType to Old type		                   }
{------------------------------------------------------------------------------}

(*$I'ErrorTextDeclarations.inc'*)

procedure GetErrorText(ErrorNumber: TErrorTypes; var ErrorString: ErrorStringType);
begin
  case ErrorTextLanguage of
    English: ErrorString:= EnglishTextsConstArr[ErrorNumber];
    Danish: ErrorString:= DanishTextsConstArr[ErrorNumber];
    (* Add additional error languages here *) 
  end; (* case *) 
end;

Procedure GetTransErrorText(VAR ErrorText: ErrorStringType; ErrorCode: WORD);
VAR
  TmpInt: INTEGER;

BEGIN
  TmpInt:= ErrorCode DIV $100;

  CASE TmpInt DIV 8 OF
      0:  GetErrorText(NoResponse,ErrorText);
      1:  GetErrorText(TimeOut,ErrorText); 
      3:  GetErrorText(WaitTooLong,ErrorText);
      4:  GetErrorText(BufferFullOrEmpty,ErrorText);
      5:  GetErrorText(DataFormatError,ErrorText);
      6:  GetErrorText(SWNOError,ErrorText);
      7:  GetErrorText(NodeAddressError, ErrorText);
      8:  GetErrorText(WriteProtection, ErrorText);
      9:  GetErrorText(InfoLengthError, ErrorText);
      10: GetErrorText(InstructionError, ErrorText);
      16: GetErrorText(ErrorDetectFailure, ErrorText);
      17: GetErrorText(OverrunFramingError, ErrorText);
      18: GetErrorText(NetShortcircuit, ErrorText);
      19: GetErrorText(PortNotMaster, ErrorText);
      20: GetErrorText(OutOfSync, ErrorText);
      21: GetErrorText(RS232HandshakeError, ErrorText)
      ELSE
        GetErrorText(UnKnownError, ErrorText); {Unknown error}
  END; (* CASE *)
END;  (* GetTransErrorText *)

PROCEDURE ConvertErrorCodeToText(ChannelType: WORD; NodeNo, ChNo: INTEGER; ErrorCode: Bit8; VAR ErrorText: ErrorStringType);
(* A nodelist element has to be inserted and updated before calling this procedure *)
VAR
  Str, Str1: ErrorStringType;
  By: BYTE;
  BytePtr: Pointer to BYTE;
  ClearError: BOOLEAN;

BEGIN (* ConvertErrorCodeToText *)
  GetErrorText(Err,Str);
  Str1:= '';

  CASE ChannelType of

    01:
    BEGIN  (* Standard service channel *)
      IF CONVERT(ErrorCode) <> 0 THEN
      BEGIN
        GetErrorText(Servicech0,Str1);
        IF ErrorCode[7] THEN
          GetErrorText(Internal_reset,Str)
        ELSE
          IF ErrorCode[6] THEN
            GetErrorText(External_reset,Str)
          ELSE
            IF ErrorCode[5] THEN
              GetErrorText(ROM_error,Str)
            ELSE
              IF ErrorCode[4] THEN
                GetErrorText(RAM_error,Str)
              ELSE
                IF ErrorCode[3] THEN
                  GetErrorText(Watchdog_error,Str)
                ELSE
                  IF ErrorCode[2] THEN
                    GetErrorText(EEPROM_error,Str)
                  ELSE
                    IF ErrorCode[1] THEN
                      GetErrorText(Battery_low,Str)
                    ELSE
                      IF ErrorCode[0] THEN
                        GetErrorText(Channelerror,Str)
      END;
    END;

    02:
    BEGIN  (* Standard digital I/O channel *)
      ClearError:= FALSE;
      GetErrorText(Digital_IO,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
      ELSE
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
          BEGIN (* Clear FlagReg to clear error *)
            GetErrorText(Overload,Str);
            ClearError:= TRUE;
          END ELSE
          IF ErrorCode[5] THEN
            GetErrorText(Underload,Str)
          ELSE
            IF ErrorCode[4] THEN
              GetErrorText(External_shortcircuit,Str)
            ELSE
              IF ErrorCode[3] THEN
                GetErrorText(Internal_shortcircuit,Str)
              ELSE
                IF ErrorCode[2] THEN
                  GetErrorText(Internal_disconnection,Str)
                ELSE
                  IF ErrorCode[1] THEN
                    GetErrorText(Feedback_error,Str)
                  ELSE
                    IF ErrorCode[0] THEN
                    BEGIN (* Clear FlagReg to clear error *)
                      GetErrorText(Programming_error,Str);
                      ClearError:= TRUE;
                    END;
          IF ClearError THEN
          BEGIN
            Disable(Error);
            BytePtr -> PointerToNode(NodeNo, ChNo*$10, 0);
            OR(BytePtr, $00);
          END;
        END;
    END;

    04:
    BEGIN  (* Standard AnalogIn channel *)
      GetErrorText(Analog_input,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
       ELSE
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
            GetErrorText(Signal_high,Str)
          ELSE
            IF ErrorCode[5] THEN
              GetErrorText(Signal_low,Str)
            ELSE
              IF ErrorCode[4] THEN
                GetErrorText(High_alarm,Str)
              ELSE
                IF ErrorCode[3] THEN
                  GetErrorText(Low_alarm,Str)
                ELSE
                  IF ErrorCode[2] THEN
                    GetErrorText(Internal_error,Str)
                  ELSE
                    IF ErrorCode[1] THEN
                      GetErrorText(External_error,Str)
        END;
    END;

    05:
    BEGIN  (* Standard AnalogOut channel *)
      GetErrorText(Analog_output,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
      ELSE
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
            GetErrorText(Signal_high,Str)
          ELSE
            IF ErrorCode[5] THEN
              GetErrorText(Signal_low,Str)
            ELSE
              IF ErrorCode[4] THEN
                GetErrorText(High_alarm,Str)
              ELSE
                IF ErrorCode[3] THEN
                  GetErrorText(Low_alarm,Str)
                ELSE
                  IF ErrorCode[2] THEN
                    GetErrorText(No_load,Str)
                  ELSE
                    IF ErrorCode[1] THEN
                      GetErrorText(Dataerror_on_setpoint,Str)
        END;
    END;

    06:
    BEGIN  (* PID channel *)
      GetErrorText(Analog_output,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
      ELSE
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
            GetErrorText(Output_high,Str)
          ELSE
            IF ErrorCode[5] THEN
              GetErrorText(Output_low,Str)
            ELSE
              IF ErrorCode[2] THEN
                GetErrorText(Dataerror_on_signal,Str)
              ELSE
                IF ErrorCode[1] THEN
                  GetErrorText(Dataerror_on_setpoint,Str)
        END;
    END;

    07:
    BEGIN  (* Calculator channel *)
      GetErrorText(CalculatorModule,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str);
    END;

    10:
    BEGIN  (* Standard weight channel *)
      GetErrorText(WeightModule,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
      ELSE
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
            GetErrorText(Signal_high,Str)
          ELSE
            IF ErrorCode[5] THEN
              GetErrorText(Signal_low,Str)
            ELSE
              IF ErrorCode[4] THEN
                GetErrorText(High_alarm,Str)
              ELSE
                IF ErrorCode[3] THEN
                  GetErrorText(Low_alarm,Str)
                ELSE
                  IF ErrorCode[2] THEN
                    GetErrorText(Internal_error,Str)
                  ELSE
                    IF ErrorCode[1] THEN
                      GetErrorText(External_error,Str)
                    ELSE
                      IF ErrorCode[0] THEN
                        GetErrorText(Overload,Str)
        END;
    END;

    11:
    BEGIN  (* Program channel *)
      GetErrorText(Prgram,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
      ELSE
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
            GetErrorText(ErrorDetectFailure,Str)
          ELSE
            IF ErrorCode[5] THEN
              GetErrorText(No_prog_after_reset,Str)
            ELSE
              IF ErrorCode[2] THEN
                GetErrorText(No_program_code,Str)
              ELSE
                IF ErrorCode[1] THEN
                  GetErrorText(Wrong_interpr_vers,Str)
                ELSE
                  IF ErrorCode[0] THEN
                    GetErrorText(Error_in_program,Str)
        END;
    END;


    12:
    BEGIN  (* Power channel *)
      GetErrorText(Power,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
      ELSE
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
            GetErrorText(Current_high,Str)
          ELSE
            IF ErrorCode[5] THEN
              GetErrorText(Voltage_high,Str)
            ELSE
              IF ErrorCode[4] THEN
                GetErrorText(High_alarm,Str)
              ELSE
                IF ErrorCode[3] THEN
                  GetErrorText(Low_alarm,Str)
                ELSE
                  IF ErrorCode[2] THEN
                    GetErrorText(Symmetry_error,Str)
                  ELSE
                    IF ErrorCode[1] THEN
                      GetErrorText(External_error,Str)
        END;
    END;

    14:
    BEGIN  (* Communication channel *)
      GetErrorText(Communication,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
      ELSE
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
            GetErrorText(Inputbuffer_overrun,Str)
          ELSE
            IF ErrorCode[5] THEN
              GetErrorText(Parity_or_framing_error,Str)
            ELSE
              IF ErrorCode[4] THEN
                GetErrorText(Reservation_timeout,Str)
              ELSE
                IF ErrorCode[3] THEN
                  GetErrorText(Config_err_datamode_param,Str)
                ELSE
                  IF ErrorCode[2] THEN
                    GetErrorText(Config_err_ChConfig,Str)
                  ELSE
                    IF ErrorCode[1] THEN
                      GetErrorText(Config_err_baudrate,Str)
                    ELSE
                      IF ErrorCode[0] THEN
                        GetErrorText(Config_err_nodeparam,Str);
        END;
    END;

    15:
    BEGIN  (* Standard digital I/O channel PD62x *)
      ClearError:= FALSE;
      GetErrorText(Digital_IO,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
      ELSE
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
          BEGIN (* Clear FlagReg to clear error *)
            GetErrorText(Overload,Str);
            ClearError:= TRUE;
          END ELSE
          IF ErrorCode[5] THEN
            GetErrorText(Underload,Str)
          ELSE
            IF ErrorCode[4] THEN
              GetErrorText(External_shortcircuit,Str)
            ELSE
              IF ErrorCode[3] THEN
                GetErrorText(Internal_shortcircuit,Str)
              ELSE
                IF ErrorCode[2] THEN
                  GetErrorText(Internal_disconnection,Str)
                ELSE
                  GetErrorText(UnKnownError,Str);
          IF ClearError THEN
          BEGIN
            Disable(Error);
            BytePtr -> PointerToNode(NodeNo, ChNo*$10, 0);
            OR(BytePtr, $00);
          END;
        END;
    END;

    56:
    BEGIN    (* PD340 Flowmeter *)
      by:= CONVERT(ErrorCode);
      GetErrorText(FlowTransmitter,Str1);
      ChNo:= 1;
      CASE by of
        $F4, $F3, $F1: GetErrorText(Reset_internal_error,Str);
        $F2          : GetErrorText(Reset_via_PNET,Str);
        $F0          : GetErrorText(Reset_powercut,Str);
        $83          : GetErrorText(PROM_Storage_error,Str);
        $82          : GetErrorText(Watchdog_error,Str);
        $81          : GetErrorText(RAM_Storage_error,Str);
        $80          : GetErrorText(Program_execution_error,Str);
        $76          : GetErrorText(EEPROM_error,Str);
        $75          : GetErrorText(RAM_or_EEPROM_error,Str);
        $64, $63, $62: GetErrorText(Temperature_connec_error,Str);
        $54          : GetErrorText(Magnetic_coil_disconnected,Str);
        $52          : GetErrorText(Magnetic_coil_Shortcircuited,Str);
        $44, $43, $42: GetErrorText(Shunt_defect,Str);
        $24          : GetErrorText(Temperature_greater_than_max,Str);
        $23          : GetErrorText(Temperature_greater_than_100C,Str);
        $08          : GetErrorText(Overrun_volume_counter_2,Str);
        $07          : GetErrorText(Overrun_volume_counter_1,Str);
        $05          : GetErrorText(Input_active,Str);
        $04          : GetErrorText(Flow_greater_than_max_or_pipe_empty,Str);
        $03          : GetErrorText(Flow_greater_than_max,Str);
        $02          : GetErrorText(Overflow_output_2,Str);
        $01          : GetErrorText(Overflow_output_3,Str);
      END;  (* CASE *)
      ErrorText:= Str1 + ': ' + Str;
    END;

    66, 70:
    BEGIN  (* Digital I/O channel in PD3100 (and NOT PD3150) *)
      GetErrorText(Digital_IO,Str1);
      IF ErrorCode[7] THEN
        GetErrorText(Module_error,Str)
      ELSE
      BEGIN
        ErrorCode:= CONVERT(CONVERT(ErrorCode) AND $74);
        IF CONVERT(ErrorCode) <> 0 THEN
        BEGIN
          IF ErrorCode[6] THEN
            GetErrorText(Overload,Str)
          ELSE
            IF ErrorCode[5] THEN
              GetErrorText(Feedback_error,Str)
            ELSE
              IF ErrorCode[4] THEN
                GetErrorText(Programming_error,Str)
              ELSE
                IF ErrorCode[2] THEN
                  GetErrorText(NetShortcircuit,Str);
        END;
      END;
    END;
  END;  (* CASE *)
  IF ChannelType <> 56 THEN
    ErrorText:= Str1 + '_' + ChNo:2:-5 + ': ' + Str;
END; (* ConvertErrorCodeToText *)

PROCEDURE WhenErrorRoutine(VAR ErrorText: ErrorStringType);
CONST
  PD340Type = WORD(56);
  PD4000Type = WORD(01);
VAR
  TransErrorText, ChannelErrorText : ErrorStringType;
  TmpErrorText: ErrorStringType;
  ChannelNo: INTEGER;
  ErrorBlock: InterFaceErrorRecord;
  ChannelTypePtr: POINTER TO WORD;
  ErrorCodePtr: POINTER TO Bit8;
  ErrorCode: Bit8;
  VarInfo: VarInfoType;

  Procedure GetVarInfo(VAR a : VarInfoType);
  BEGIN
    PCodeCall(10 : $CB00,
                   $FC00,
                   a,
                   $BA23);
  END;


BEGIN (* WhenErrorRoutine *)
  WHILE NOT BufferEmpty(InterFaceErrorBuffer) DO
    ErrorBlock:= InterFaceErrorBuffer;
        (* Read an element from the InterFaceErrorBuffer *)
  ChannelNo:= ErrorBlock.VarAddress DIV $10;
  tmpErrorText:= VarName(ErrorBlock.SWNo);
  ChannelErrorText:=VarName(ErrorBlock.SWNo, ChannelNo);
  IF tmpErrorText <> ChannelErrorText THEN
    tmpErrorText:= tmpErrorText + ' ' + ChannelErrorText;

  IF ((ErrorBlock.ErrorCode AND $0009) <> 0) THEN   (* TrmError *)
  BEGIN
    GetTransErrorText(TransErrorText, ErrorBlock.ErrorCode);
    tmpErrorText:= tmpErrorText + ' ' + TransErrorText;
  END
  ELSE
  BEGIN   (* Error in module *)
    VarInfo.SWNo:=ErrorBlock.SWNo;
    GetVarInfo(VarInfo);
    NodeList[WhenErrorNodeElement].DeviceType:= VarInfo.DeviceType;
    NodeList[WhenErrorNodeElement].Code:= 2; (* NoBit *)
    NodeList[WhenErrorNodeElement].NodeAddr:= VarInfo.PNETAddr;
    IF ((VarInfo.DeviceType = 340) OR (VarInfo.DeviceType = 4000)) THEN
    BEGIN
      IF (VarInfo.DeviceType = 4000) THEN
        ChannelTypePtr -> PD4000Type
      ELSE
        ChannelTypePtr -> PD340Type;
      ErrorCodePtr -> PointerToNode(WhenErrorNodeElement, 3,0);
      ErrorCode:= ErrorCodePtr;
    END
    ELSE
    BEGIN
      ChannelTypePtr -> PointerToNode(WhenErrorNodeElement, ChannelNo*$10 + $0E,0);
      ErrorCodePtr -> PointerToNode(WhenErrorNodeElement, ChannelNo*$10 + $0F,0);
      ErrorCode:= ErrorCodePtr;
      IF (ChannelTypePtr <> 1) AND ErrorCode[7] THEN 
              (* not a service channel and Module_Error *)
      BEGIN   (* point to the service channel to read the module error code *)
        ChannelTypePtr -> PointerToNode(WhenErrorNodeElement, $0E,0);
        ErrorCodePtr -> PointerToNode(WhenErrorNodeElement, $0F,0);
        ErrorCode:= ErrorCodePtr;
      END;
    END;

    ConvertErrorCodeToText(ChannelTypePtr, WhenErrorNodeElement, ChannelNo,
                           ErrorCode, TransErrorText);
    tmpErrorText:= tmpErrorText + ' ' + TransErrorText;
  END;
  ErrorText:= TmpErrorText; (* Assign the Errortext to the GlobalErrorString *)
  WHILE NOT BufferEmpty(InterFaceErrorBuffer) DO
    ErrorBlock:= InterFaceErrorBuffer;
        (* clear the InterFaceErrorBuffer *)
END; (* WhenErrorRoutine *)
